!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center
! Land Information System Framework (LISF)
! Version 7.3
!
! Copyright (c) 2020 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !ROUTINE: readcrd_pptEnsFcst
! \label{readcrd_pptEnsFcst}
!
! !REVISION HISTORY:
! 26 Sept 2016; KR Arsenault, Initial Code developed
!
! !INTERFACE:    
subroutine readcrd_pptEnsFcst()
! !USES:
  use ESMF
  use LIS_coreMod, only : LIS_rc, LIS_config
  use LIS_logMod,  only : LIS_logunit, LIS_verify, LIS_endrun
  use pptEnsFcst_forcingMod, only : pptensfcst_struc
!
! !DESCRIPTION:
!
!  This routine reads the options specific to the 
!  meteorological forcing data generated by LDT or 
!  by outside routines/scripts and written
!  in Netcdf format.
!  
!EOP
  implicit none

  integer :: n, rc

  call ESMF_ConfigFindLabel(LIS_config,"Precipitation ensemble forecast directory:",rc=rc)
  call ESMF_ConfigGetAttribute(LIS_config,pptensfcst_struc%directory,rc=rc)

  call ESMF_ConfigFindLabel(LIS_config,"Precipitation ensemble forecast number of ensemble members:",rc=rc)
  call LIS_verify(rc, 'Precipitation ensemble forecast number of ensemble members: not defined')
  call ESMF_ConfigGetAttribute(LIS_config,pptensfcst_struc%max_ens_members,rc=rc)
!  do n=1,LIS_rc%nnest
!     call ESMF_ConfigGetAttribute(LIS_config,pptensfcst_struc(n)%max_ens_members,rc=rc)
!  enddo

  write(LIS_logunit,*) "[INFO] "
  write(LIS_logunit,*) "[INFO] Using the Precip Ens Fcst forcing dataset"
  write(LIS_logunit,*) "[INFO] Precipitation ensemble forecast directory: ",&
                       trim(pptensfcst_struc%directory)
  write(LIS_logunit,*) "[INFO] Precipitation ensemble forecast number of members: ",&
                       pptensfcst_struc%max_ens_members

end subroutine readcrd_pptEnsFcst

